 DEFINE PAGE00, SPACE=ROM
 SEGMENT PAGE00

 public Reboot, Boot
 extern PutChar, DrawString, KeyboardInit, MeasureString
 extern LCDInit, ClearBuffer, LCDDelay, FastCopy
 extern WaitForever, GetKey, WaitKey, FlushKeys
 extern OSVersion, TitleScreen, FSInit
 extern ResetTextLocation, Sleep

 nolist
 include "KnightOS.inc"
 list

	jr Boot
  	db 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
 	db 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
	db 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0			; 55 zeroes
	jr SysInterrupt
 	db 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
	db 0,0,0,0,0,0								; 27 zeroes
	jp Boot
	db 0FFh, 0FFh, 5Ah, 0A5h
Boot:	; 0058h
	; This is called as soon as the batteries hit the calc,
	; not when ON is pressed
	call FSInit
	; Sleep requires memory to be properly arranged (hence FSInit)
	; and requires the stack to be initialized
	ld sp, 0D000h		; Initialize the stack
	call Sleep
Reboot:
	ld iy, FlagStart	; Initialize flags
	
	;Get the calculator model
	in a, (2)
 	rlca ;Roll bit 7 into carry.
	jr nc, Regular83Plus
	and 40h ;Test bit 5, now rotated into bit 6.
	jr z, Silver83Plus
	in a, (21h)
	and 03h
	jr z, Regular84Plus
	; 84+ SE
	ld a, 3
	jr CheckDone
	Silver83Plus:
	ld a, 1
	jr CheckDone
	Regular84Plus:
	ld a, 2
	jr CheckDone
	Regular83Plus:
	ld a, 0
	CheckDone:
	ld (ModelID), a

	; Load drivers
	call LCDInit
	call KeyboardInit

	; This is where the OS start-up code goes.
	; Delete it and replace it with your own boot code.
	call ClearBuffer
	call ResetTextLocation
	ld hl, MsgBootSuccess
	call DrawString
	call FastCopy
	call FlushKeys
	call WaitKey
	call FlushKeys
	jr Boot

MsgBootSuccess:
	db "Boot success.  Press any\nkey to shut down.", 0

; System Interrupt Routines
SysInterrupt:
	exx
	ex af, af'
	in a, (04h)
	bit 0, a
	jr nz, HandleON
	bit 1, a
	jr nz, HandleTimer1
	bit 2, a
	jr nz, HandleTimer2
	bit 4, a
	jr nz, HandleLink
	jr InterruptDone
HandleON:
	in a, (03h)
	res 0, a
	out (03h), a
	set 0, a
	out (03h), a
	; ON interrupt
	jr InterruptDone
HandleTimer1:
	in a, (03h)
	res 1, a
	out (03h), a
	set 1, a
	out (03h), a
	; Timer one interrupt (might be best to merge with timer 2)
	jr InterruptDone
HandleTimer2:
	in a, (03h)
	res 2, a
	out (03h), a
	set 2, a
	out (03h), a
	; Timer two interrupt
	jr InterruptDone
HandleLink:
	in a, (03h)
	res 4, a
	out (03h), a
	set 4, a
	out (03h), a
	; Link interrupt
InterruptDone:
	ex af, af'
	exx
	ei
	ret
